@@ -22,7 +22,7 @@ import deployv_static
 from extend_me import Extensible
 from psycopg2 import OperationalError
 from datetime import datetime
-from docker import APIClient as Client
+from docker import from_env
 from docker.errors import APIError, NullResource, NotFound, ImageNotFound
 from deployv.base.extensions_core import events, load_extensions
 from deployv.base import postgresv, nginxv, errors, clonev
@@ -103,23 +103,23 @@ class CommandV(Extensible):
         deactivate command is called or if the `deactivating_database`
         parameter is specified in the config json > group_config dictionary.
         """
-        return self.__config.deactivating_database
+        return self.__config.deactivating_database or False
 
     def pushing_images(self):
         """Transitions condition used to finish the process after pushing
         the images. This condition was only added in order to keep the
         posibility to create a `push` command.
         """
-        return self.__config.pushing_images
+        return self.__config.pushing_images or False
 
     def create_database(self):
         """Transitions condition used to add the `restore` command to
         the process. This method returns the value stored in the
         config json > instance > restore_db key.
         """
-        if self.__config.reloading_config:
+        if self.__config.reloading_config or self.__config.container_config.get('keep_db'):
             return False
-        return self.__config.instance_config.get('restore_db')
+        return self.__config.instance_config.get('restore_db') or False
 
     def need_revert(self):
         """Transitions condition used to go to the `Create container` state
@@ -127,7 +127,7 @@ class CommandV(Extensible):
         to revert the changes if an error occurs when the production databases
         are updated.
         """
-        return self._need_revert
+        return self._need_revert or False
 
     def deactivate_required(self):
         """Transitions condition used to add the `deactivate` step to the
@@ -159,7 +159,7 @@ class CommandV(Extensible):
         deactivate_backup command is called or if the `deactivating_backup`
         key is specified in the config json > group_config dictionary.
         """
-        return self.__config.deactivating_backup
+        return self.__config.deactivating_backup or False
 
     def build_image_required(self):
         """Transitons condition used to add the `build_image` step to
@@ -167,7 +167,7 @@ class CommandV(Extensible):
         `full_stack` keys in config json > container_config are True.
         """
         cfg = self.__config.container_config
-        return cfg.get('build_image') or cfg.get('full_stack')
+        return cfg.get('build_image') or cfg.get('full_stack') or False
 
     def skip_push(self):
         """Transitions condition used to skip the `push_images` step during
@@ -184,7 +184,7 @@ class CommandV(Extensible):
         is called or if the config json > group_config > create_container key
         is True.
         """
-        return self.__config.create_container
+        return self.__config.create_container or False
 
     def updating_instance(self):
         """Transitions condition used to take the process steps required to
@@ -194,17 +194,21 @@ class CommandV(Extensible):
 
     def keep_database(self):
         """Transitions condition used to know where keep the database or
-        the instance is updating and continue with the deactivate of database.
+        the instance is updating and continue with the deactivation of database.
         """
         if self.__config.reloading_config:
             return False
         return self.__config.container_config.get('keep_db') or self.__config.updating_instance
 
+    def is_production(self):
+        """Transition condition, used to know if instance to deploy is production or not"""
+        return self.__config.instance_config.get('instance_type') == 'production'
+
     def reloading_config(self):
         """Transitions condition used to know which steps to take when reloading
         a container's config.
         """
-        return self.__config.reloading_config
+        return self.__config.reloading_config or False
 
     def use_btrfs(self):
         """Transtions condition used to know how to handle the databases (delete, restore, backup)
@@ -218,13 +222,32 @@ class CommandV(Extensible):
         instance_type = self.__config.instance_config.get('instance_type')
         return self.__config.backup_type == 'zfs' and instance_type != 'production'
 
-    def _can_add_new(self):
+    def container_created(self):
+        """Transitions condition used to know if the Odoo server is created.
+        """
+        if self.__instance_manager.docker_id:
+            return True
+        return False
+
+    def test_instance_created(self):
+        """Transitions condition used to go to the `Executing final steps` state
+        when is a test instance and the state before is `Creating container`
+        """
+        instance_type = self.__config.instance_config.get('instance_type')
+        return self.container_created() and instance_type == 'test'
+
+    def _can_add_new(self, container_name=''):
         """As we have a limited amount of resources in each node we need to check if the running
         instances count is less than the allowed to run a new one.
 
         :returns: If the count of running instances is lower than the allowed max.
         :rtype: bool
         """
+        # container_names is required due to result obtained from cli.containers()
+        if container_name:
+            container_names = [container_name, '/' + container_name]
+        else:
+            container_names = []
         containers = self.instance_manager.cli.containers()
         max_instances = self.__config.deployer.get('max_instances')
         if not max_instances:
@@ -232,7 +255,7 @@ class CommandV(Extensible):
         count = 0
         for container_info in containers:
             for name in container_info.get('Names'):
-                if re.match(r'/([\w\d\_\-]+)_odoo$', name):
+                if re.match(r'/([\w\d\_\-\.]+)_odoo$', name) and name not in container_names:
                     count += 1
                     break
         logger.debug('Instance count %s/%s', count, max_instances)
@@ -303,7 +326,8 @@ class CommandV(Extensible):
                'attachments': list()}
         self.sender.send_message('Creating instance.')
         self.__config.check_config('create')
-        if not self._can_add_new():
+        container_name = self.__config.container_config.get('container_name')
+        if not self._can_add_new(container_name):
             logger.info('Max instance count reached (%s allowed)',
                         self.__config.deployer.get('max_instances'))
             res.update({'error': 'Reached max instance count for this node ({} allowed)'
@@ -323,11 +347,10 @@ class CommandV(Extensible):
             res.update({'error': info.get('error')})
             self.send_message_error(res)
             return res
-        file_name = self.instance_manager.save_repositories('post_process.json')
-        post_branch = os.path.join(self.__config.temp_working_folder, file_name)
+        post_branch = self.instance_manager.save_repositories('post_process.json')
         res.get('attachments').append(
             {
-                'file_name': file_name,
+                'file_name': os.path.basename(post_branch),
                 'file': utils.generate_attachment(post_branch),
                 'type': 'application/json'
             }
@@ -345,8 +368,17 @@ class CommandV(Extensible):
                         'nginx_url': "http://{url}".format(url=nginx_url),
                         'instance_log': '{url}/logs/odoo_stdout.log'.format(url=nginx_url)
                     })
-        if not self.create_database():
-            self.instance_manager.start_instance()
+        params = self.__config.instance_config.get('restore_filestore')
+        if params:
+            logger.info("Restoring filestore")
+            filestore_res = self.__instance_manager.restore_filestore(
+                params.get('path'), params.get('db_name'))
+            if filestore_res.get('error'):
+                res.update({'error': filestore_res.get('error')})
+                self.send_message_error(res)
+                return res
+            logger.info("Finish restore filestore")
+        self.instance_manager.start_instance()
         res.update({'instance_revert': self._need_revert})
         self.sender.send_message('Instance created', body=res)
         self.create_done_trigger()
@@ -370,12 +402,19 @@ class CommandV(Extensible):
         extra_config = {
             'nginx_url': self.instance_manager.config.get('nginx_url'),
             'use_mailhog': self.__config.instance_config.get("use_mailhog")}
-        self.instance_manager.deactivate_database(database, extra_config)
+        deactivate_res = self.instance_manager.deactivate_database(database, extra_config)
+        if deactivate_res.get('error'):
+            self.instance_manager._skip_install_test_ribbon = True
+            self.send_message_error(deactivate_res)
+            return deactivate_res
         new_admin = db_config.get('admin', False)
         if new_admin:
             db_config.update({'db_name': database})
             admin_id = self.get_admin_id(db_config=db_config)
             instancev.InstanceV.change_password(admin_id, new_admin, database, db_config)
+        # Start Odoo after deactivated the instance.
+        if self.__instance_manager.docker_id:
+            self.__instance_manager.start_instance()
         self.sender.send_message('Database {} deactivated.'.format(database))
         self.deactivate_done_trigger()
 
@@ -531,9 +570,12 @@ class CommandV(Extensible):
         if res.get('error', False):
             self.send_message_error(res)
             return res
-        self.__instance_manager.start_instance()
+        # Wait start the Odoo after deactivate the isntance unless it is a `production` type.
+        if (self.__instance_manager.docker_id and
+                self.instance_manager.instance_type == 'production'):
+            self.__instance_manager.start_instance()
         logger.debug('restore_res: %s', str(restore_res))
-        if restore_from:
+        if restore_from and not self.__instance_manager.docker_id:
             utils.clean_files(self.__config.backup_src)
         user = self.instance_manager.get_odoo_users(database_name,
                                                     user_id=self.get_admin_id())
@@ -636,7 +678,12 @@ class CommandV(Extensible):
         self.instance_manager.set_parameters(database_name, params)
         self.new_deactivate(db_config)
         bash_lines = ["chown -R odoo:odoo /home/odoo/.local"]
-        self.instance_manager.execute_in_auxiliary_container(database_name, bash_lines, volumes)
+        cmd_res = self.instance_manager.execute_in_auxiliary_container(database_name,
+                                                                       bash_lines, volumes)
+        if cmd_res.get('error'):
+            res.update(cmd_res)
+            self.send_message_error(res)
+            return res
         self.sender.send_message(msg='Database "{}" created.'.format(database_name), body=res)
         self.restore_done_trigger()
         return res
@@ -714,7 +761,10 @@ class CommandV(Extensible):
             logger.error(restore_res[1])
             return False
         logger.info('Deactivating database')
-        self.instance_manager.deactivate_database(db_name)
+        deactivate_res = self.instance_manager.deactivate_database(db_name)
+        if deactivate_res.get('error'):
+            self.send_message_error(deactivate_res)
+            return deactivate_res
         admin_id = self.get_admin_id(db_config=db_config)
         instancev.InstanceV.change_password(admin_id, db_config.get('admin'), db_name, db_config)
         dump_path = 'dump.sql' if not self.__config.jobs else 'database_dump'
@@ -847,7 +897,8 @@ class CommandV(Extensible):
         self.__config.check_config('updatedb')
         res = {'command': 'updatedb', 'must_parse': True}
         db_name = db_name or self.__config.instance_config.get('config').get('db_name')
-        if self.__config.instance_config.get('instance_type') == 'updates':
+        if (self.__config.instance_config.get('instance_type') == 'updates' and
+                self.__config.instance_config.get('auto_compare')):
             self.get_db_data(db_name, 'pre-update')
         self.sender.send_message('Updating database "{}".'.format(db_name))
         try:
@@ -893,7 +944,8 @@ class CommandV(Extensible):
                 'base': self.instance_manager.config_variables.get('docker_image_repo')
                 or self.__config.container_config.get('customer_image'),
                 'tag_latest': bool(instance_type == 'updates')})
-        if self.__config.instance_config.get('instance_type') == 'updates':
+        if (self.__config.instance_config.get('instance_type') == 'updates' and
+                self.__config.instance_config.get('auto_compare')):
             self.get_db_data(db_name, 'post-update')
         self._update_done = True
         self.sender.send_message('Database "{}" updated.'.format(db_name), body=res)
@@ -903,7 +955,7 @@ class CommandV(Extensible):
     def _stop_instance(self):
         try:
             self.__instance_manager.stop_instance()
-        except NotFound as error:
+        except (errors.CommandError, NotFound) as error:
             logger.debug('Error message %s', utils.get_error_message(error))
             utils.get_error_message(error)
             return 'Instance already deleted'
@@ -1014,11 +1066,11 @@ class CommandV(Extensible):
         :return: Containers (as dict) with the domain and mapping ports.
         :rtype: list
         """
-        cli = Client()
+        cli = from_env().api
         containers = cli.containers()
         res = list()
         logger.debug('Geting containes names')
-        regex = r'/([\w\d\_\-]+)_odoo$'
+        regex = r'/([\w\d\_\-\.]+)_odoo$'
         for container_info in containers:
             for name in container_info.get('Names'):
                 logger.debug('Name : %s', name)
@@ -1054,7 +1106,7 @@ class CommandV(Extensible):
         :rtype: dict
         """
         containers_config = self.get_containers()
-        nginx_manager = nginxv.NginxV(self.__config.deployer.get('nginx_folder'))
+        nginx_manager = nginxv.NginxV(self.__config)
         nginx_manager.update_sites(containers_config)
         nginx_manager.restart()
         return containers_config
@@ -1277,12 +1329,7 @@ class CommandV(Extensible):
             'update_logs': start_result.get('result').get('update_logs'),
             'result_type': 'pre-backup'
         })
-        odoo_home = self.__config.container_config.get('env_vars').get('odoo_home')
-        self.__instance_manager.exec_cmd(
-            'bash -c "branchesv save -p {home}/instance -f /tmp/repositories.json"'
-            .format(home=odoo_home)
-        )
-        path_branches = os.path.join(self.instance_manager.temp_folder, 'repositories.json')
+        path_branches = self.instance_manager.save_repositories("repositories.json")
         start_result.get('attachments').append({
             'file_name': os.path.basename(path_branches),
             'file': utils.generate_attachment(path_branches),
@@ -1462,13 +1509,8 @@ class CommandV(Extensible):
     def _switch_production_container(self, container_name):
         res = {'result': False}
         manager = self.instance_manager
-        odoo_home = self.__config.container_config.get('env_vars').get('odoo_home')
         # Get the info of the repos
-        cmd_output = manager.exec_cmd(
-            ('bash -c "branchesv save -p {home}/instance -f /tmp/repositories.json"')
-            .format(home=odoo_home)
-        )
-        logger.debug('Branchesv result:\n%s', cmd_output)
+        self.__instance_manager.save_repositories("repositories.json")
         # Stop instance
         logger.info('Stopping instance in "%s"', container_name)
         self.sender.send_message('Stopping instance')
@@ -1551,6 +1593,20 @@ class CommandV(Extensible):
         """
         utils.clean_files(self.__config.temp_working_folder)
 
+    @events
+    def verify_quantity_instances(self):
+        """Verify that max limit of deployed instances won't be exceeding"""
+        res = {'command': 'create', 'result': {}}
+        container_name = self.__config.container_config.get('container_name')
+        if not self._can_add_new(container_name):
+            msg = 'Reached max instance count for this node ({} allowed)'.format(
+                self.__config.deployer.get('max_instances'))
+            logger.error(msg)
+            res.update({'error': msg})
+            self.send_message_error(res)
+            raise Exception(msg)
+        return {}
+
     def commit_history(self):
         """ Downloads the repositories if they do not exist, and if the do
         it pulls the latest changes, gets the different commits between two commits
@@ -1583,7 +1639,10 @@ class CommandV(Extensible):
         extra_config = {
             'nginx_url': self.instance_manager.config.get('nginx_url'),
             'use_mailhog': self.__config.instance_config.get("use_mailhog")}
-        self.instance_manager.deactivate_database(database, extra_config)
+        deactivate_res = self.instance_manager.deactivate_database(database, extra_config)
+        if deactivate_res.get('error'):
+            self.send_message_error(deactivate_res)
+            return deactivate_res
         new_admin = db_config.get('admin', False)
         if new_admin:
             db_config.update({'db_name': database})
