@@ -59,8 +59,8 @@ Where:
 """
 
 import logging
-from os import path
-from docker import DockerClient
+from os import path, environ
+from docker import DockerClient, from_env
 from docker import APIClient as Client
 import docker.errors
 import simplejson as json
@@ -116,8 +116,12 @@ class DockerV:
     __url = None
 
     def __init__(self, config, timeout=300, docker_url="unix://var/run/docker.sock"):
-        self.__cli = Client(base_url=docker_url, timeout=timeout)
-        self.__cli2 = DockerClient(base_url=docker_url, timeout=timeout)
+        if environ.get("DOCKER_HOST"):
+            self.__cli = from_env(timeout=timeout).api
+            self.__cli2 = from_env(timeout=timeout)
+        else:
+            self.__cli = Client(base_url=docker_url, timeout=timeout)
+            self.__cli2 = DockerClient(base_url=docker_url, timeout=timeout)
         self.__config = config
         self.__url = docker_url
         if self.__config.get('id', False):
@@ -181,6 +185,9 @@ class DockerV:
         """
         return self.__docker_id.lower() if self.__docker_id else None
 
+    def unset_docker_id(self):
+        self.__docker_id = False
+
     @property
     def docker_env(self):
         """Gets env vars from a docker container.
@@ -843,7 +850,7 @@ class DockerV:
         """
         image = image or self.config.get('image_name')
         if not image:
-            return
+            return {}
         self.pull(image)
         volumes = volumes or self.config.get('volumes')
         env_vars = self.docker_env
@@ -859,9 +866,18 @@ class DockerV:
                 'Waiting for the command to finish, current state: %s', cont.status)
             sleep(5)
             cont.reload()
-        res = cont.logs()
+        cmd_res = cont.logs(stdout=True, stderr=True)
+        info = self.inspect(cont.short_id)
+        exit_code = info.get('State').get('ExitCode')
+        logger.debug("Command result: %s, Exit Code: %s", cmd_res, exit_code)
         cont.remove()
-        return res and res.decode()
+        if exit_code == 0:
+            res = {'result': cmd_res.decode()}
+        else:
+            error = 'Command %(cmd)s failed with exit code: %(code)s %(error)s' % {
+                "cmd": command, "code": exit_code, "error": cmd_res.decode()}
+            res = {'error': error}
+        return res
 
     def get_env_from_image(self, image=False):
         """Get the environment of a specific image or use the image set in the config,
