@@ -67,8 +67,11 @@ class PostgresConnector:
         :rtype: dict or bool
         """
         raw = kwargs.pop('raw', False)
-        change = sql_str.lower().startswith((
-            'insert into', 'update', 'create', 'delete', 'alter', 'reassign', 'grant'))
+        sql_sentences = (
+            'insert into', 'update', 'create', 'delete', 'alter', 'reassign', 'grant', 'security',)
+        if kwargs.pop('with_as_write', False):
+            sql_sentences += ('with', )
+        change = sql_str.lower().startswith(sql_sentences)
         try:
             logger.debug('SQL: %s', sql_str)
             self.__cursor.execute(sql_str, *args)
