@@ -5,7 +5,7 @@ import spur
 import re
 import docker
 import shutil
-from docker import APIClient as Client
+from docker import from_env
 from deployv.helpers import utils
 from deployv.base.errors import BuildError, NoItemFound
 from deployv.instance import ODOO_BINARY
@@ -29,7 +29,7 @@ class BuildImage:
 
             :param config: Config Config object with the format used by deployv.
         """
-        self.client = Client()
+        self.client = from_env().api
         self._config = config
         self._config.tag = self._config.tag or self._generate_image_name()
         self._files_folder = path.join(self._config.temp_working_folder, 'files')
