@@ -91,7 +91,7 @@ from passlib.context import CryptContext
 from deployv.base.dockerv import DockerV
 from deployv.base.postgresv import PostgresShell, PostgresConnector
 from deployv.base import errors
-from deployv.helpers import utils, container, database_helper, json_helper
+from deployv.helpers import utils, container, database_helper, json_helper, pg_anon
 from deployv.instance import ODOO_BINARY, PSQL_VERSIONS, TRAVIS_VERSIONS
 from datetime import datetime
 from jinja2 import Template
@@ -110,6 +110,7 @@ class InstanceV(DockerV):
 
     def __init__(self, config, timeout=300, docker_url="unix://var/run/docker.sock"):
         self.__full_config = config
+        self._skip_install_test_ribbon = False
         container_config = config.container_config
         logger.debug('Container config : %s',
                      json.dumps(container_config, sort_keys=True, indent=2))
@@ -137,14 +138,13 @@ class InstanceV(DockerV):
 
         :return: A dict with the basic info of container
         """
-        logger.debug('Psql in container: %s', self.config.get('postgres_container', False))
         # if it will be a full stack container all the parameters are changed (no matter what is
         # specified in the config) to target the local env and added the nginx, postgres and ssh
         # ports
         if self.config.get('full_stack', False):
             self.config.get('ports').update({'22': None})
+            self.update_info_postgres_container()
         self.pull()
-        self.start_postgres_container()
         logger.info('Starting Odoo container %s',
                     self.config.get('container_name'))
         self.config.update({'container_hostname': container.generate_hostname(self.__full_config)})
@@ -216,8 +216,7 @@ class InstanceV(DockerV):
     def add_authorized_keys(self):
         tmp = self.__full_config.temp_folder or '/tmp'
         git_cli = github.Github()
-        users = ['nhomar', 'ruiztulio', 'moylop260', 'josemoralesp', 'oscarolar'] +\
-            self.__full_config.instance_config.get('authorized_users', [])
+        users = self.__full_config.instance_config.get('authorized_users', [])
         authorized_keys_path = path.join(tmp, 'authorized_keys')
         authorized_keys_tar_path = path.join(tmp, 'keys.tar')
         with open(authorized_keys_path, 'w') as authorized_keys:
@@ -479,11 +478,14 @@ class InstanceV(DockerV):
         working_folder = path.join(self.config.get("working_folder"),
                                    self.config.get('container_name'))
         db_config = self.db_config
+        entrypoint_exists = self.exec_cmd_image(
+            'test -x /entry_point', self.config.get('volumes'), working_folder, image=image_name)
+        entrypoint = '/entry_point' if not entrypoint_exists.get('error') else '/entry_point.py'
         update_command = (
-            "/entry_point.py run --user odoo '/home/odoo/instance/odoo/{odoo}"
+            "{entrypoint} run --user odoo '/home/odoo/instance/odoo/{odoo}"
             " -c /home/odoo/.openerp_serverrc"
             " -d {database} -u {{module}} --logfile {{log}} --stop-after-init'"
-            .format(odoo=self.odoo_binary, database=db_name))
+            .format(entrypoint=entrypoint, odoo=self.odoo_binary, database=db_name))
         pg_shell = PostgresShell(utils.odoo2postgres(db_config))
         for module in modules:
             for attempt in range(2):
@@ -539,7 +541,11 @@ class InstanceV(DockerV):
         working_folder = path.join(self.config.get("working_folder"),
                                    self.config.get('container_name'))
         logger.info('Updating database %s using COU', db_name)
-        update_command = "/entry_point.py cou -d {database}".format(database=db_name)
+        entrypoint_exists = self.exec_cmd_image(
+            'test -x /entry_point', self.config.get('volumes'), working_folder, image=image_name)
+        entrypoint = '/entry_point' if not entrypoint_exists.get('error') else '/entry_point.py'
+        update_command = "{entrypoint} cou -d {database}".format(
+            entrypoint=entrypoint, database=db_name)
         local_log_path = path.expanduser(
             path.join(working_folder, 'tmp/deployvlogs/cou_update.log'))
         pg_shell = PostgresShell(utils.odoo2postgres(self.db_config))
@@ -673,16 +679,8 @@ class InstanceV(DockerV):
                 self.__full_config.backup_type in ['btrfs', 'zfs']):
             return {}
         destination = '/var/log/pg_log'
-        if self.config.get('keep_db') or self.__full_config.reloading_config:
-            info = self.inspect(self.config.get('postgres_container_name'))
-            db_host = info.get('NetworkSettings').get('Gateway')
-            self.config.update({'pg_log_postgres': self.get_specify_volumen(info, destination)})
-            db_port = self.get_db_port(info)
-            self.__full_config.instance_config.get("config").update({
-                'db_host': db_host, 'db_port': db_port})
-            self.config.get('env_vars').update({
-                'db_port': db_port,
-                'db_host': db_host, 'external_db_port': db_port})
+        res = self.update_info_postgres_container()
+        if res:
             return {}
         ins_config = self.config.copy()
         psql_version = self.get_postgres_version()
@@ -729,6 +727,7 @@ class InstanceV(DockerV):
         # Don't expose the 5432 port in the odoo container
         # if a postgres container was created
         self.config.get('ports').pop('5432', False)
+        self.unset_docker_id()
         return res
 
     def get_postgres_version(self):
@@ -769,16 +768,16 @@ class InstanceV(DockerV):
 
     def restore_database(self, db_name=False, backup_dir=False):
         res = {'result': {}}
-        if not self.docker_id:
-            res.update({
-                'error': 'The container %s does not exist' % self.config.get('container_name')
-            })
-            return res
+        error = False
         db_name_config = self.__full_config.instance_config.get('config').get('db_name')
         customer_id = (self.__full_config.instance_config.get('customer_id') or
                        self.docker_env and self.docker_env.get("customer"))
         if not customer_id:
-            return {'error': 'No such customer to restore the database'}
+            error = 'No such customer to restore the database'
+        if error:
+            return {'error': error}
+        logger.debug('Psql in container: %s', self.config.get('postgres_container', False))
+        self.start_postgres_container()
         use_template = self.__full_config.container_config.get('use_template')
         jobs = self.__full_config.jobs or 5
         helper_obj = database_helper.DatabaseHelper.get_helper(use_template)
@@ -802,14 +801,21 @@ class InstanceV(DockerV):
         filestore = path.join(candidate, 'filestore')
         db_name = db_name or db_name_config or utils.generate_dbname(
             self.__full_config, path.basename(res_search[1]), self.__full_config.prefix)
-        self.stop_instance()
+        if self.docker_id:
+            self.stop_instance()
         create_res = db_helper.create_database(candidate, db_name, self.db_config.get('db_owner'),
                                                self.db_config.get('db_owner_passwd'), jobs=jobs)
         if not create_res[0]:
             res.update({'error': create_res[1]})
             return res
         create_date = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
-        self.restore_filestore(filestore, db_name)
+        if self.docker_id and path.exists(filestore):
+            filestore_res = self.restore_filestore(filestore, db_name)
+            if filestore_res.get('error'):
+                return filestore_res
+        else:
+            self.__full_config.instance_config.update({
+                'restore_filestore': {'path': filestore, 'db_name': db_name}})
         res.get('result').update({'database_name': db_name, 'create_date': create_date})
         self.create_extension_unaccent(db_name)
         return res
@@ -824,7 +830,6 @@ class InstanceV(DockerV):
         :returns: List of the files that will be compressed in the backup
         """
         files = []
-        odoo_home = self.config.get('env_vars').get('odoo_home')
         jobs = self.__full_config.jobs
         postgres_cfg = self.db_config
         data_dir = data_dir or postgres_cfg.get('data_dir')
@@ -869,12 +874,7 @@ class InstanceV(DockerV):
         else:
             logger.info('There is not attachments folder to backup')
         try:
-            cmd_output = self.exec_cmd(
-                ('bash -c "branchesv save -p {home}/instance -f /tmp/repositories.json"')
-                .format(home=odoo_home)
-            )
-            logger.debug('Branchesv result:\n%s', cmd_output)
-            json_repos = path.join(self.temp_folder, 'repositories.json')
+            json_repos = self.save_repositories("repositories.json")
             if path.exists(json_repos):
                 files.append((json_repos, 'repositories.json'))
         except errors.CommandError as error:
@@ -999,7 +999,8 @@ class InstanceV(DockerV):
             "chown -R odoo:odoo /home/odoo/.local",
             "chmod -R o+r {fsname}"
         ]
-        self.execute_in_auxiliary_container(database_name, bash_lines, volumes)
+        res = self.execute_in_auxiliary_container(database_name, bash_lines, volumes)
+        return res
 
     def execute_in_auxiliary_container(self, db_name, commands, volumes=False):
         """Method that creates an auxiliary container copy of the
@@ -1011,24 +1012,37 @@ class InstanceV(DockerV):
 
         :param db_name: Name of the database, use to know where the volumes
             created by deployv are.
-        :type db: str
+        :type db_name: str
         :param commands: List of commands to be executed
         :type commands: list
-        :return: None
+        :return: Dictionary with the `result` key (list) if all the commands are
+            executed, and/or the `error` key (list) if any or all of them failed.
         """
+        res = {}
+        results = []
+        errors_list = []
         logger.debug('Getting properties')
         fs_name = path.join('/home/odoo/.local/share/Odoo/filestore/', db_name)
         logger.debug('Creating auxiliary container')
-        cmd = ' && '.join(commands)
-        cmd = cmd.format(fsname=fs_name)
-        try:
-            container_cfg = self.inspect()
-        except errors.NoSuchContainer:
-            container_cfg = {}
-        image = container_cfg.get('Config', {}).get('Image') or self.config.get('image_name')
-        self.exec_cmd_image("bash -c '%s'" % (cmd), volumes, image=image)
+        for cmd in commands:
+            cmd = cmd.format(fsname=fs_name)
+            try:
+                container_cfg = self.inspect()
+            except errors.NoSuchContainer:
+                container_cfg = {}
+            image = container_cfg.get('Config', {}).get('Image') or self.config.get('image_name')
+            cmd_res = self.exec_cmd_image("bash -c '%s'" % (cmd), volumes, image=image)
+            if cmd_res.get('error'):
+                errors_list.append(cmd_res['error'])
+            elif cmd_res.get('result'):
+                results.append(cmd_res['result'])
+        if results:
+            res.update({'result': '\n'.join(results)})
+        if errors_list:
+            res.update({'error': '\n'.join(errors_list)})
+        return res
 
-    def execute_query(self, db_config, query, allow_errors=False):
+    def execute_query(self, db_config, query, allow_errors=False, with_as_write=False):
         """Executes the provided query upon the specified database.
 
         :param db_config: Name of the database where the query will be executed,
@@ -1036,13 +1050,16 @@ class InstanceV(DockerV):
         :param allow_errors: If True, the errors raised regarding missing or already
             existing tables while executing the query will be ignored, a
             psycopg2.ProgrammingError will be raised otherwise.
+        :param with_as_write: indicates if with sentence should be interpreted
+            as a changing sentence (like insert, update), and return True when executed
+            instead of return a list of records (which happens for select query)
         """
         allowed_errors = ([] if not allow_errors else
                           ['relation', 'does not exist', 'already exists',
                            'no existe', 'ya existe'])
         with PostgresConnector(db_config) as db:
             try:
-                res = db.execute(query)
+                res = db.execute(query, with_as_write=with_as_write)
             except psycopg2.ProgrammingError as error:
                 error_msg = utils.get_error_message(error)
                 if any([string in error_msg for string in allowed_errors]):
@@ -1542,14 +1559,20 @@ class InstanceV(DockerV):
             res_cmd = self.exec_cmd_image(cmd, {name: '/tmp'}, wf_path)
         except errors.CommandError:
             return res
-        if not res_cmd or 'No such file or directory' in res_cmd:
+        if not res_cmd.get('result') or 'No such file or directory' in res_cmd.get('result'):
             logger.info('Could not find the deactivate.jinja file in %s', odoo_home)
             return res
-        for file_sql in res_cmd.splitlines():
+        if res_cmd.get('error'):
+            return res_cmd
+        for file_sql in res_cmd['result'].splitlines():
             if not file_sql:
                 continue
             sqls = self.exec_cmd_image("cat {0}".format(file_sql), {name: '/tmp'}, wf_path)
-            sql_json = json_helper.load_json(Template(sqls).render(**env_vars))
+            if sqls.get('error'):
+                logger.error(sqls.get('error'))
+                res.update(sqls)
+                return res
+            sql_json = json_helper.load_json(Template(sqls['result']).render(**env_vars))
             res.update(sql_json or {})
         return res
 
@@ -1594,6 +1617,76 @@ class InstanceV(DockerV):
         logger.info('Container started %s', container_id)
         return container_id
 
+    def create_vx_schema(self, db_config):
+        """Create vx schema and required functions to anonymize data
+        :return: boolean, error_message (True when process is successful)"""
+        try:
+            self.execute_query(
+                db_config, "CREATE SCHEMA vx; SECURITY LABEL FOR anon ON SCHEMA vx IS 'TRUSTED';")
+        except Exception as e:
+            error = utils.get_error_message(e)
+            logger.error('Error while create vx schema, and set security label: %s', error)
+            return False, error
+        queries = pg_anon.get_queries_vx_functions()
+        try:
+            self.execute_query(db_config, queries)
+        except Exception as e:
+            error = utils.get_error_message(e)
+            logger.error('Error while create functions in vx schema: %s', error)
+            return False, error
+        return True, ''
+
+    def anonymize_db(self, db_config):
+        """Apply masked functions to anonymize some data, as indicated by config file.
+        Verify that postgresql anonymizer is installed, otherwise, try to install it.
+
+        :param db_config: Configuration for database where the query will be executed
+        :return: boolean, error_message (True when process is successful)"""
+        # verify if postgresql anonymizer extension is available
+        res = self.execute_query(
+            db_config, "SELECT name, installed_version FROM pg_catalog.pg_available_extensions "
+                       "WHERE name='anon'")
+        if res:
+            if res[0]['installed_version'] is None:
+                query = "ALTER DATABASE {} SET session_preload_libraries = 'anon';".format(
+                    db_config['dbname'])
+                try:
+                    self.execute_query(db_config, query)
+                except Exception as e:
+                    error = utils.get_error_message(e)
+                    logger.error('Error while set anon extension: %s', error)
+                    return False, 'Anonymizer error: {}'.format(error)
+                query = "CREATE EXTENSION anon CASCADE; SELECT anon.init(); " \
+                        "ALTER DATABASE {} SET anon.restrict_to_trusted_schemas = on;" \
+                    .format(db_config['dbname'])
+                try:
+                    self.execute_query(db_config, query)
+                except Exception as e:
+                    error = utils.get_error_message(e)
+                    logger.error('Error while create anon extension: %s', error)
+                    return False, 'Anonymizer error: {}'.format(error)
+
+            res = self.execute_query(db_config,
+                                     "SELECT COUNT(*) as qty FROM pg_catalog.pg_namespace "
+                                     "WHERE nspname='vx';")
+            if not res[0]['qty']:
+                res, error = self.create_vx_schema(db_config)
+                if not res:
+                    return res, 'Anonymizer error: {}'.format(error)
+
+            queries = pg_anon.get_queries_from_operations_file(
+                '/etc/deployv/deactivate/anon_operations.csv')
+            for item in queries:
+                try:
+                    self.execute_query(db_config, item)
+                except Exception as e:
+                    error = utils.get_error_message(e)
+                    logger.error('Error while executing anonymize data query: %s', error)
+                    return False, 'Anonymizer error: {}'.format(error)
+            return True, ''
+        logger.error('Extension postgresql anonymizer is not available')
+        return False, 'Anonymizer error: Extension postgresql anonymizer is not available'
+
     def deactivate_database(self, db_name, extra_config=False):
         """Executes the deactivation queries found in the deactivation.jinja file
         on the provided database.
@@ -1601,8 +1694,12 @@ class InstanceV(DockerV):
         :param db_name: Name of the database where the queries will be executed
         :param extra_config: Dictionary with the additional config used to
             render the deactivation.jinja file
+
+        :return: Error message when data anonymization fails, None otherwise
         """
         logger.info('Deactivating database %s', db_name)
+        if self.docker_id:
+            self.stop_instance()
         db_config_user = self.db_config.copy()
         db_config_user.update({'db_name': db_name})
         db_config_owner = db_config_user.copy()
@@ -1612,8 +1709,22 @@ class InstanceV(DockerV):
         })
         db_config_user = utils.odoo2postgres(db_config_user)
         db_config_owner = utils.odoo2postgres(db_config_owner)
+        # executes anonymization, if required
+        if path.isfile('/etc/deployv/deactivate/anon_operations.csv'):
+            logger.info('Start anonymization of data')
+            res, error_msg = self.anonymize_db(db_config_user)
+            if res:
+                logger.info('Successful anonymization of data')
+            else:
+                logger.exception('Failed anonymization of data')
+                postgres = PostgresShell(utils.odoo2postgres(self.db_config.copy()))
+                postgres.drop(db_name)
+                return {'error': error_msg}
         extra_config = extra_config or {}
         extra_sqls = self.extras_deactivate()
+        if extra_sqls.get('error'):
+            logger.error(extra_sqls.get('error'))
+            return extra_sqls
         query_config = ("select id from ir_model_fields where name"
                         "= 'value' and model = 'ir.config_parameter';")
         config_parameter_id = self.execute_query(db_config_user, query_config)
@@ -1638,13 +1749,14 @@ class InstanceV(DockerV):
                 logger.debug('Running %s', name)
                 sql = sqls[name]
                 try:
-                    self.execute_query(db_config_user, sql, allow_errors=True)
+                    self.execute_query(db_config_user, sql, allow_errors=True, with_as_write=True)
                 except psycopg2.errors.InsufficientPrivilege as error:
                     logger.warning("Inssufficient permissions for user '%s': %s",
                                    db_config_user.get('user'), utils.get_error_message(error))
                     self.execute_query(db_config_owner, sql, allow_errors=True)
         except psycopg2.OperationalError as error:
             logger.exception('Could not deactivate the database: %s', error.message)
+        return {}
 
     @property
     def db_config(self):
@@ -1667,7 +1779,11 @@ class InstanceV(DockerV):
             'db_password': res.get('db_password') or env_vars.get('db_password'),
             'data_dir': self.get_data_dir()
         })
-        if utils.is_docker_ip(res.get('db_host')) or res.get('db_host') == 'localhost':
+        docker_network_info = self.cli.inspect_network('bridge')
+        docker_gateway = docker_network_info.get('IPAM').get('Config')[0].get('Gateway')
+        is_docker_ip = utils.is_docker_ip(res.get('db_host'), docker_gateway)
+        if ((is_docker_ip or res.get('db_host') == 'localhost') and not
+                self.__full_config.deployer.get('deployv_in_docker')):
             res.update({'db_host': '127.0.0.1'})
         return res
 
@@ -1690,13 +1806,23 @@ class InstanceV(DockerV):
         :param file_name: The file's name where the repositories's information are saved.
         :type: str
         """
+        if not self.temp_folder:
+            logger.warning("Save Repositories: Doesn't exist a temp folder where save the file")
+            return ''
         home = self.__full_config.container_config.get('env_vars').get('odoo_home')
+        odoo_user = self.__full_config.container_config.get('env_vars').get('odoo_user')
         filename = filename or 'branches.json'
-        command = ('bash -c "branchesv save -p %(home)s/instance -f /tmp/%(name)s"' %
-                   {'home': home, 'name': filename})
-        name = path.basename(self.__full_config.temp_working_folder)
-        self.exec_cmd_image(command, {name: '/tmp'}, self.__full_config.working_folder)
-        return filename
+        self.exec_cmd('bash -c "rm -rf /tmp/%s"' % (filename))
+        cmd = 'bash -c "branchesv save -p %s/instance -f /tmp/%s"' % (home, filename)
+        file_path = path.join(self.temp_folder, filename)
+        cmd_output = self.exec_cmd(cmd)
+        logger.debug('Branchesv result:\n%s', cmd_output)
+        if not path.exists(file_path):
+            cmd = 'su %s -c "branchesv save -p %s/instance -f /tmp/%s"' % (
+                odoo_user, home, filename)
+            cmd_output = self.exec_cmd(cmd)
+            logger.debug('Branchesv result:\n%s', cmd_output)
+        return file_path
 
     @property
     def config_variables(self):
@@ -1810,6 +1936,23 @@ print(json.dumps({
         })
         return utils.odoo2postgres(admin_config)
 
+    def update_info_postgres_container(self):
+        """Update the env_vars with the port and host where is running the Postgres container.
+        """
+        destination = '/var/log/pg_log'
+        if self.config.get('keep_db') or self.__full_config.reloading_config:
+            info = self.inspect(self.config.get('postgres_container_name'))
+            db_host = info.get('NetworkSettings').get('Gateway')
+            self.config.update({'pg_log_postgres': self.get_specify_volumen(info, destination)})
+            db_port = self.get_db_port(info)
+            self.__full_config.instance_config.get("config").update({
+                'db_host': db_host, 'db_port': db_port})
+            self.config.get('env_vars').update({
+                'db_port': db_port,
+                'db_host': db_host, 'external_db_port': db_port})
+            return True
+        return False
+
 
 class DummyInstanceV:
 
