@@ -31,6 +31,7 @@ TRIGGER_MAPPINGS = {
     'deactivate_backup': 'deactivate_backup_trigger',
     'push_image': 'push_images_trigger',
     'update_production': 'update_prod_trigger',
+    'compare_databases': 'compare_databases',
 }
 
 
@@ -114,6 +115,8 @@ class BaseWorker:
         if not parameters.get('container_config', {}).get('domain'):
             parameters.get('container_config', {}).update({
                 'domain': self.config_obj.deployer.get('domain')})
+        action_name = 'regenerate' if command_name == 'delete_instance' and parameters.get(
+            'group_config').get('create_container') else command_name
         default_params = {
             'backup_src': self.config_obj.deployer.get('backup_folder'),
             'temp_folder': self.config_obj.deployer.get('temp_folder'),
@@ -128,7 +131,7 @@ class BaseWorker:
             'res_model': message.res_model,
             'model': message.model,
             'user_id': message.user_id,
-            'command': 'orchestv.{mo}.{co}'.format(mo=module, co=command_name),
+            'action_name': action_name,
             'response_to': message.message_id,
         }
         load_defaults = parameters.get('group_config', {}).get('load_defaults', False)
